package java.security.cert;

import sun.security.x509.GeneralSubtree;
import sun.security.x509.GeneralSubtrees;
import sun.security.x509.PolicyInformation;
import sun.security.x509.GeneralNames;
import sun.security.x509.GeneralName;
import sun.security.x509.AlgorithmId;
import java.util.Arrays;
import sun.security.x509.ExtendedKeyUsageExtension;
import sun.security.x509.CertificatePoliciesExtension;
import sun.security.x509.SubjectAlternativeNameExtension;
import sun.security.x509.PrivateKeyUsageExtension;
import sun.security.util.DerInputStream;
import sun.security.x509.X509CertImpl;
import sun.misc.HexDumpEncoder;
import sun.security.x509.CertificatePolicyId;
import java.util.Vector;
import sun.security.x509.OIDName;
import sun.security.x509.IPAddressName;
import sun.security.x509.URIName;
import sun.security.x509.EDIPartyName;
import sun.security.x509.X400Address;
import sun.security.x509.DNSName;
import sun.security.x509.RFC822Name;
import sun.security.x509.OtherName;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Collections;
import java.util.HashSet;
import sun.security.x509.X509Key;
import sun.security.util.DerValue;
import java.io.IOException;
import sun.security.x509.X500Name;
import sun.security.x509.NameConstraintsExtension;
import sun.security.x509.CertificatePolicySet;
import sun.security.x509.GeneralNameInterface;
import java.util.List;
import java.util.Set;
import java.security.PublicKey;
import java.util.Date;
import javax.security.auth.x500.X500Principal;
import java.math.BigInteger;
import sun.security.util.ObjectIdentifier;
import sun.security.util.Debug;

public class X509CertSelector implements CertSelector
{
    private static final Debug debug;
    private static final ObjectIdentifier ANY_EXTENDED_KEY_USAGE;
    private BigInteger serialNumber;
    private X500Principal issuer;
    private X500Principal subject;
    private byte[] subjectKeyID;
    private byte[] authorityKeyID;
    private Date certificateValid;
    private Date privateKeyValid;
    private ObjectIdentifier subjectPublicKeyAlgID;
    private PublicKey subjectPublicKey;
    private byte[] subjectPublicKeyBytes;
    private boolean[] keyUsage;
    private Set<String> keyPurposeSet;
    private Set<ObjectIdentifier> keyPurposeOIDSet;
    private Set<List<?>> subjectAlternativeNames;
    private Set<GeneralNameInterface> subjectAlternativeGeneralNames;
    private CertificatePolicySet policy;
    private Set<String> policySet;
    private Set<List<?>> pathToNames;
    private Set<GeneralNameInterface> pathToGeneralNames;
    private NameConstraintsExtension nc;
    private byte[] ncBytes;
    private int basicConstraints;
    private X509Certificate x509Cert;
    private boolean matchAllSubjectAltNames;
    private static final Boolean FALSE;
    private static final int PRIVATE_KEY_USAGE_ID = 0;
    private static final int SUBJECT_ALT_NAME_ID = 1;
    private static final int NAME_CONSTRAINTS_ID = 2;
    private static final int CERT_POLICIES_ID = 3;
    private static final int EXTENDED_KEY_USAGE_ID = 4;
    private static final int NUM_OF_EXTENSIONS = 5;
    private static final String[] EXTENSION_OIDS;
    static final int NAME_ANY = 0;
    static final int NAME_RFC822 = 1;
    static final int NAME_DNS = 2;
    static final int NAME_X400 = 3;
    static final int NAME_DIRECTORY = 4;
    static final int NAME_EDI = 5;
    static final int NAME_URI = 6;
    static final int NAME_IP = 7;
    static final int NAME_OID = 8;
    
    public X509CertSelector() {
        this.basicConstraints = -1;
        this.matchAllSubjectAltNames = true;
    }
    
    public void setCertificate(X509Certificate x509Cert) {
        this.x509Cert = x509Cert;
    }
    
    public void setSerialNumber(BigInteger serialNumber) {
        this.serialNumber = serialNumber;
    }
    
    public void setIssuer(X500Principal issuer) {
        this.issuer = issuer;
    }
    
    public void setIssuer(String dname) throws IOException {
        if (dname == null) {
            this.issuer = null;
        }
        else {
            this.issuer = new X500Name(dname).asX500Principal();
        }
    }
    
    public void setIssuer(byte[] name) throws IOException {
        try {
            this.issuer = ((name == null) ? null : new X500Principal(name));
        }
        catch (IllegalArgumentException cause) {
            throw new IOException("Invalid name", cause);
        }
    }
    
    public void setSubject(X500Principal subject) {
        this.subject = subject;
    }
    
    public void setSubject(String dname) throws IOException {
        if (dname == null) {
            this.subject = null;
        }
        else {
            this.subject = new X500Name(dname).asX500Principal();
        }
    }
    
    public void setSubject(byte[] name) throws IOException {
        try {
            this.subject = ((name == null) ? null : new X500Principal(name));
        }
        catch (IllegalArgumentException cause) {
            throw new IOException("Invalid name", cause);
        }
    }
    
    public void setSubjectKeyIdentifier(byte[] array) {
        if (array == null) {
            this.subjectKeyID = null;
        }
        else {
            this.subjectKeyID = (byte[])array.clone();
        }
    }
    
    public void setAuthorityKeyIdentifier(byte[] array) {
        if (array == null) {
            this.authorityKeyID = null;
        }
        else {
            this.authorityKeyID = (byte[])array.clone();
        }
    }
    
    public void setCertificateValid(Date date) {
        if (date == null) {
            this.certificateValid = null;
        }
        else {
            this.certificateValid = (Date)date.clone();
        }
    }
    
    public void setPrivateKeyValid(Date date) {
        if (date == null) {
            this.privateKeyValid = null;
        }
        else {
            this.privateKeyValid = (Date)date.clone();
        }
    }
    
    public void setSubjectPublicKeyAlgID(String oid) throws IOException {
        if (oid == null) {
            this.subjectPublicKeyAlgID = null;
        }
        else {
            this.subjectPublicKeyAlgID = new ObjectIdentifier(oid);
        }
    }
    
    public void setSubjectPublicKey(PublicKey subjectPublicKey) {
        if (subjectPublicKey == null) {
            this.subjectPublicKey = null;
            this.subjectPublicKeyBytes = null;
        }
        else {
            this.subjectPublicKey = subjectPublicKey;
            this.subjectPublicKeyBytes = subjectPublicKey.getEncoded();
        }
    }
    
    public void setSubjectPublicKey(byte[] array) throws IOException {
        if (array == null) {
            this.subjectPublicKey = null;
            this.subjectPublicKeyBytes = null;
        }
        else {
            this.subjectPublicKeyBytes = (byte[])array.clone();
            this.subjectPublicKey = X509Key.parse(new DerValue(this.subjectPublicKeyBytes));
        }
    }
    
    public void setKeyUsage(boolean[] array) {
        if (array == null) {
            this.keyUsage = null;
        }
        else {
            this.keyUsage = (boolean[])array.clone();
        }
    }
    
    public void setExtendedKeyUsage(Set<String> c) throws IOException {
        if (c == null || c.isEmpty()) {
            this.keyPurposeSet = null;
            this.keyPurposeOIDSet = null;
        }
        else {
            this.keyPurposeSet = Collections.unmodifiableSet(new HashSet(c));
            this.keyPurposeOIDSet = new HashSet();
            Iterator<String> iterator = this.keyPurposeSet.iterator();
            while (iterator.hasNext()) {
                this.keyPurposeOIDSet.add(new ObjectIdentifier((String)iterator.next()));
            }
        }
    }
    
    public void setMatchAllSubjectAltNames(boolean matchAllSubjectAltNames) {
        this.matchAllSubjectAltNames = matchAllSubjectAltNames;
    }
    
    public void setSubjectAlternativeNames(Collection<List<?>> collection) throws IOException {
        if (collection == null) {
            this.subjectAlternativeNames = null;
            this.subjectAlternativeGeneralNames = null;
        }
        else {
            if (collection.isEmpty()) {
                this.subjectAlternativeNames = null;
                this.subjectAlternativeGeneralNames = null;
                return;
            }
            Set<List<?>> cloneAndCheckNames = cloneAndCheckNames(collection);
            this.subjectAlternativeGeneralNames = parseNames(cloneAndCheckNames);
            this.subjectAlternativeNames = cloneAndCheckNames;
        }
    }
    
    public void addSubjectAlternativeName(int n, String s) throws IOException {
        this.addSubjectAlternativeNameInternal(n, s);
    }
    
    public void addSubjectAlternativeName(int n, byte[] array) throws IOException {
        this.addSubjectAlternativeNameInternal(n, array.clone());
    }
    
    private void addSubjectAlternativeNameInternal(int i, Object o) throws IOException {
        GeneralNameInterface generalNameInterface = makeGeneralNameInterface(i, o);
        if (this.subjectAlternativeNames == null) {
            this.subjectAlternativeNames = new HashSet();
        }
        if (this.subjectAlternativeGeneralNames == null) {
            this.subjectAlternativeGeneralNames = new HashSet();
        }
        ArrayList list = new ArrayList(2);
        list.add(Integer.valueOf(i));
        list.add(o);
        this.subjectAlternativeNames.add(list);
        this.subjectAlternativeGeneralNames.add(generalNameInterface);
    }
    
    private static Set<GeneralNameInterface> parseNames(Collection<List<?>> collection) throws IOException {
        HashSet<GeneralNameInterface> set = new HashSet();
        for (List list : collection) {
            if (list.size() != 2) {
                throw new IOException("name list size not 2");
            }
            Object value = list.get(0);
            if (!(value instanceof Integer)) {
                throw new IOException("expected an Integer");
            }
            set.add(makeGeneralNameInterface((int)value, list.get(1)));
        }
        return set;
    }
    
    static boolean equalNames(Collection<?> collection, Collection<?> collection2) {
        if (collection == null || collection2 == null) {
            return collection == collection2;
        }
        return collection.equals(collection2);
    }
    
    static GeneralNameInterface makeGeneralNameInterface(int i, Object obj) throws IOException {
        if (X509CertSelector.debug != null) {
            X509CertSelector.debug.println("X509CertSelector.makeGeneralNameInterface(" + i + ")...");
        }
        RFC822Name rfc822Name;
        if ((obj instanceof String)) {
            if (X509CertSelector.debug != null) {
                X509CertSelector.debug.println("X509CertSelector.makeGeneralNameInterface() name is String: " + obj);
            }
            switch (i) {
                case 1: {
                    rfc822Name = new RFC822Name((String)obj);
                    break;
                }
                case 2: {
                    rfc822Name = new DNSName((String)obj);
                    break;
                }
                case 4: {
                    rfc822Name = new X500Name((String)obj);
                    break;
                }
                case 6: {
                    rfc822Name = new URIName((String)obj);
                    break;
                }
                case 7: {
                    rfc822Name = new IPAddressName((String)obj);
                    break;
                }
                case 8: {
                    rfc822Name = new OIDName((String)obj);
                    break;
                }
                default: {
                    throw new IOException("unable to parse String names of type " + i);
                }
            }
            if (X509CertSelector.debug != null) {
                X509CertSelector.debug.println("X509CertSelector.makeGeneralNameInterface() result: " + rfc822Name.toString());
            }
        }
        else if ((obj instanceof byte[])) {
            DerValue derValue = new DerValue((byte[])(byte[])obj);
            if (X509CertSelector.debug != null) {
                X509CertSelector.debug.println("X509CertSelector.makeGeneralNameInterface() is byte[]");
            }
            switch (i) {
                case 0: {
                    rfc822Name = new OtherName(derValue);
                    break;
                }
                case 1: {
                    rfc822Name = new RFC822Name(derValue);
                    break;
                }
                case 2: {
                    rfc822Name = new DNSName(derValue);
                    break;
                }
                case 3: {
                    rfc822Name = new X400Address(derValue);
                    break;
                }
                case 4: {
                    rfc822Name = new X500Name(derValue);
                    break;
                }
                case 5: {
                    rfc822Name = new EDIPartyName(derValue);
                    break;
                }
                case 6: {
                    rfc822Name = new URIName(derValue);
                    break;
                }
                case 7: {
                    rfc822Name = new IPAddressName(derValue);
                    break;
                }
                case 8: {
                    rfc822Name = new OIDName(derValue);
                    break;
                }
                default: {
                    throw new IOException("unable to parse byte array names of type " + i);
                }
            }
            if (X509CertSelector.debug != null) {
                X509CertSelector.debug.println("X509CertSelector.makeGeneralNameInterface() result: " + rfc822Name.toString());
            }
        }
        else {
            if (X509CertSelector.debug != null) {
                X509CertSelector.debug.println("X509CertSelector.makeGeneralName() input name not String or byte array");
            }
            throw new IOException("name not String or byte array");
        }
        return rfc822Name;
    }
    
    public void setNameConstraints(byte[] value) throws IOException {
        if (value == null) {
            this.ncBytes = null;
            this.nc = null;
        }
        else {
            this.ncBytes = (byte[])value.clone();
            this.nc = new NameConstraintsExtension(X509CertSelector.FALSE, value);
        }
    }
    
    public void setBasicConstraints(int basicConstraints) {
        if (basicConstraints < -2) {
            throw new IllegalArgumentException("basic constraints less than -2");
        }
        this.basicConstraints = basicConstraints;
        return;
    }
    
    public void setPolicy(Set<String> c) throws IOException {
        if (c == null) {
            this.policySet = null;
            this.policy = null;
        }
        else {
            Set<Object> unmodifiableSet = Collections.unmodifiableSet(new HashSet(c));
            Iterator<Object> iterator = unmodifiableSet.iterator();
            Vector<CertificatePolicyId> ids = new Vector();
            while (iterator.hasNext()) {
                Object next = iterator.next();
                if (!(next instanceof String)) {
                    throw new IOException("non String in certPolicySet");
                }
                ids.add(new CertificatePolicyId(new ObjectIdentifier((String)next)));
            }
            this.policySet = unmodifiableSet;
            this.policy = new CertificatePolicySet(ids);
        }
    }
    
    public void setPathToNames(Collection<List<?>> collection) throws IOException {
        if (collection == null || collection.isEmpty()) {
            this.pathToNames = null;
            this.pathToGeneralNames = null;
        }
        else {
            Set<List<?>> cloneAndCheckNames = cloneAndCheckNames(collection);
            this.pathToGeneralNames = parseNames(cloneAndCheckNames);
            this.pathToNames = cloneAndCheckNames;
        }
    }
    
    void setPathToNamesInternal(Set<GeneralNameInterface> pathToGeneralNames) {
        this.pathToNames = Collections.emptySet();
        this.pathToGeneralNames = pathToGeneralNames;
    }
    
    public void addPathToName(int n, String s) throws IOException {
        this.addPathToNameInternal(n, s);
    }
    
    public void addPathToName(int n, byte[] array) throws IOException {
        this.addPathToNameInternal(n, array.clone());
    }
    
    private void addPathToNameInternal(int i, Object o) throws IOException {
        GeneralNameInterface generalNameInterface = makeGeneralNameInterface(i, o);
        if (this.pathToGeneralNames == null) {
            this.pathToNames = new HashSet();
            this.pathToGeneralNames = new HashSet();
        }
        ArrayList list = new ArrayList(2);
        list.add(Integer.valueOf(i));
        list.add(o);
        this.pathToNames.add(list);
        this.pathToGeneralNames.add(generalNameInterface);
    }
    
    public X509Certificate getCertificate() {
        return this.x509Cert;
    }
    
    public BigInteger getSerialNumber() {
        return this.serialNumber;
    }
    
    public X500Principal getIssuer() {
        return this.issuer;
    }
    
    public String getIssuerAsString() {
        return (this.issuer == null) ? null : this.issuer.getName();
    }
    
    public byte[] getIssuerAsBytes() throws IOException {
        return (this.issuer == null) ? null : this.issuer.getEncoded();
    }
    
    public X500Principal getSubject() {
        return this.subject;
    }
    
    public String getSubjectAsString() {
        return (this.subject == null) ? null : this.subject.getName();
    }
    
    public byte[] getSubjectAsBytes() throws IOException {
        return (this.subject == null) ? null : this.subject.getEncoded();
    }
    
    public byte[] getSubjectKeyIdentifier() {
        if (this.subjectKeyID == null) {
            return null;
        }
        return (byte[])this.subjectKeyID.clone();
    }
    
    public byte[] getAuthorityKeyIdentifier() {
        if (this.authorityKeyID == null) {
            return null;
        }
        return (byte[])this.authorityKeyID.clone();
    }
    
    public Date getCertificateValid() {
        if (this.certificateValid == null) {
            return null;
        }
        return (Date)this.certificateValid.clone();
    }
    
    public Date getPrivateKeyValid() {
        if (this.privateKeyValid == null) {
            return null;
        }
        return (Date)this.privateKeyValid.clone();
    }
    
    public String getSubjectPublicKeyAlgID() {
        if (this.subjectPublicKeyAlgID == null) {
            return null;
        }
        return this.subjectPublicKeyAlgID.toString();
    }
    
    public PublicKey getSubjectPublicKey() {
        return this.subjectPublicKey;
    }
    
    public boolean[] getKeyUsage() {
        if (this.keyUsage == null) {
            return null;
        }
        return (boolean[])this.keyUsage.clone();
    }
    
    public Set<String> getExtendedKeyUsage() {
        return this.keyPurposeSet;
    }
    
    public boolean getMatchAllSubjectAltNames() {
        return this.matchAllSubjectAltNames;
    }
    
    public Collection<List<?>> getSubjectAlternativeNames() {
        if (this.subjectAlternativeNames == null) {
            return null;
        }
        return cloneNames(this.subjectAlternativeNames);
    }
    
    private static Set<List<?>> cloneNames(Collection<List<?>> collection) {
        try {
            return cloneAndCheckNames(collection);
        }
        catch (IOException ex) {
            throw new RuntimeException("cloneNames encountered IOException: " + ex.getMessage());
        }
    }
    
    private static Set<List<?>> cloneAndCheckNames(Collection<List<?>> collection) throws IOException {
        HashSet set = new HashSet();
        Iterator<Object> iterator = collection.iterator();
        while (iterator.hasNext()) {
            set.add(new ArrayList((List)iterator.next()));
        }
        for (List list : set) {
            if (list.size() != 2) {
                throw new IOException("name list size not 2");
            }
            Object value = list.get(0);
            if (!(value instanceof Integer)) {
                throw new IOException("expected an Integer");
            }
            int intValue = (int)value;
            if (intValue < 0 || intValue > 8) {
                throw new IOException("name type not 0-8");
            }
            Object value2 = list.get(1);
            if (!(value2 instanceof byte[]) && !(value2 instanceof String)) {
                if (X509CertSelector.debug != null) {
                    X509CertSelector.debug.println("X509CertSelector.cloneAndCheckNames() name not byte array");
                }
                throw new IOException("name not byte array or String");
            }
            else if ((value2 instanceof byte[])) {
                list.set(1, ((byte[])(byte[])value2).clone());
            }
            else {
                continue;
            }
        }
        return set;
    }
    
    public byte[] getNameConstraints() {
        if (this.ncBytes == null) {
            return null;
        }
        return (byte[])this.ncBytes.clone();
    }
    
    public int getBasicConstraints() {
        return this.basicConstraints;
    }
    
    public Set<String> getPolicy() {
        return this.policySet;
    }
    
    public Collection<List<?>> getPathToNames() {
        if (this.pathToNames == null) {
            return null;
        }
        return cloneNames(this.pathToNames);
    }
    
    @Override
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("X509CertSelector: [\n");
        if (this.x509Cert != null) {
            sb.append("  Certificate: " + this.x509Cert.toString() + "\n");
        }
        if (this.serialNumber != null) {
            sb.append("  Serial Number: " + this.serialNumber.toString() + "\n");
        }
        if (this.issuer != null) {
            sb.append("  Issuer: " + this.getIssuerAsString() + "\n");
        }
        if (this.subject != null) {
            sb.append("  Subject: " + this.getSubjectAsString() + "\n");
        }
        sb.append("  matchAllSubjectAltNames flag: " + String.valueOf(this.matchAllSubjectAltNames) + "\n");
        if (this.subjectAlternativeNames != null) {
            sb.append("  SubjectAlternativeNames:\n");
            for (List list : this.subjectAlternativeNames) {
                sb.append("    type " + list.get(0) + ", name " + list.get(1) + "\n");
            }
        }
        if (this.subjectKeyID != null) {
            sb.append("  Subject Key Identifier: " + new HexDumpEncoder().encodeBuffer(this.subjectKeyID) + "\n");
        }
        if (this.authorityKeyID != null) {
            sb.append("  Authority Key Identifier: " + new HexDumpEncoder().encodeBuffer(this.authorityKeyID) + "\n");
        }
        if (this.certificateValid != null) {
            sb.append("  Certificate Valid: " + this.certificateValid.toString() + "\n");
        }
        if (this.privateKeyValid != null) {
            sb.append("  Private Key Valid: " + this.privateKeyValid.toString() + "\n");
        }
        if (this.subjectPublicKeyAlgID != null) {
            sb.append("  Subject Public Key AlgID: " + this.subjectPublicKeyAlgID.toString() + "\n");
        }
        if (this.subjectPublicKey != null) {
            sb.append("  Subject Public Key: " + this.subjectPublicKey.toString() + "\n");
        }
        if (this.keyUsage != null) {
            sb.append("  Key Usage: " + keyUsageToString(this.keyUsage) + "\n");
        }
        if (this.keyPurposeSet != null) {
            sb.append("  Extended Key Usage: " + this.keyPurposeSet.toString() + "\n");
        }
        if (this.policy != null) {
            sb.append("  Policy: " + this.policy.toString() + "\n");
        }
        if (this.pathToGeneralNames != null) {
            sb.append("  Path to names:\n");
            Iterator<GeneralNameInterface> iterator2 = this.pathToGeneralNames.iterator();
            while (iterator2.hasNext()) {
                sb.append("    " + iterator2.next() + "\n");
            }
        }
        sb.append("]");
        return sb.toString();
    }
    
    private static String keyUsageToString(boolean[] array) {
        String s = "KeyUsage [\n";
        try {
            if (array[0]) {
                s += "  DigitalSignature\n";
            }
            if (array[1]) {
                s += "  Non_repudiation\n";
            }
            if (array[2]) {
                s += "  Key_Encipherment\n";
            }
            if (array[3]) {
                s += "  Data_Encipherment\n";
            }
            if (array[4]) {
                s += "  Key_Agreement\n";
            }
            if (array[5]) {
                s += "  Key_CertSign\n";
            }
            if (array[6]) {
                s += "  Crl_Sign\n";
            }
            if (array[7]) {
                s += "  Encipher_Only\n";
            }
            if (array[8]) {
                s += "  Decipher_Only\n";
            }
        }
        catch (ArrayIndexOutOfBoundsException ex) {}
        return s + "]\n";
    }
    
    private static Extension getExtensionObject(X509Certificate x509Certificate, int n) throws IOException {
        if ((x509Certificate instanceof X509CertImpl)) {
            X509CertImpl x509CertImpl = (X509CertImpl)x509Certificate;
            switch (n) {
                case 0: {
                    return x509CertImpl.getPrivateKeyUsageExtension();
                }
                case 1: {
                    return x509CertImpl.getSubjectAlternativeNameExtension();
                }
                case 2: {
                    return x509CertImpl.getNameConstraintsExtension();
                }
                case 3: {
                    return x509CertImpl.getCertificatePoliciesExtension();
                }
                case 4: {
                    return x509CertImpl.getExtendedKeyUsageExtension();
                }
                default: {
                    return null;
                }
            }
        }
        else {
            byte[] extensionValue = x509Certificate.getExtensionValue(X509CertSelector.EXTENSION_OIDS[n]);
            if (extensionValue == null) {
                return null;
            }
            byte[] octetString = new DerInputStream(extensionValue).getOctetString();
            switch (n) {
                case 0: {
                    try {
                        return new PrivateKeyUsageExtension(X509CertSelector.FALSE, octetString);
                    }
                    catch (CertificateException ex) {
                        throw new IOException(ex.getMessage());
                    }
                }
                case 1: {
                    return new SubjectAlternativeNameExtension(X509CertSelector.FALSE, octetString);
                }
                case 2: {
                    return new NameConstraintsExtension(X509CertSelector.FALSE, octetString);
                }
                case 3: {
                    return new CertificatePoliciesExtension(X509CertSelector.FALSE, octetString);
                }
                case 4: {
                    return new ExtendedKeyUsageExtension(X509CertSelector.FALSE, octetString);
                }
                default: {
                    return null;
                }
            }
        }
    }
    
    @Override
    public boolean match(Certificate certificate) {
        if (!(certificate instanceof X509Certificate)) {
            return false;
        }
        X509Certificate x509Certificate = (X509Certificate)certificate;
        if (X509CertSelector.debug != null) {
            X509CertSelector.debug.println("X509CertSelector.match(SN: " + x509Certificate.getSerialNumber().toString(16) + "\n  Issuer: " + x509Certificate.getIssuerDN() + "\n  Subject: " + x509Certificate.getSubjectDN() + ")");
        }
        if (this.x509Cert != null && !this.x509Cert.equals(x509Certificate)) {
            if (X509CertSelector.debug != null) {
                X509CertSelector.debug.println("X509CertSelector.match: certs don't match");
            }
            return false;
        }
        else if (this.serialNumber != null && !this.serialNumber.equals(x509Certificate.getSerialNumber())) {
            if (X509CertSelector.debug != null) {
                X509CertSelector.debug.println("X509CertSelector.match: serial numbers don't match");
            }
            return false;
        }
        else if (this.issuer != null && !this.issuer.equals(x509Certificate.getIssuerX500Principal())) {
            if (X509CertSelector.debug != null) {
                X509CertSelector.debug.println("X509CertSelector.match: issuer DNs don't match");
            }
            return false;
        }
        else if (this.subject != null && !this.subject.equals(x509Certificate.getSubjectX500Principal())) {
            if (X509CertSelector.debug != null) {
                X509CertSelector.debug.println("X509CertSelector.match: subject DNs don't match");
            }
            return false;
        }
        else {
            if (this.certificateValid != null) {
                try {
                    x509Certificate.checkValidity(this.certificateValid);
                }
                catch (CertificateException ex) {
                    if (X509CertSelector.debug != null) {
                        X509CertSelector.debug.println("X509CertSelector.match: certificate not within validity period");
                    }
                    return false;
                }
            }
            if (this.subjectPublicKeyBytes != null && !Arrays.equals(this.subjectPublicKeyBytes, x509Certificate.getPublicKey().getEncoded())) {
                if (X509CertSelector.debug != null) {
                    X509CertSelector.debug.println("X509CertSelector.match: subject public keys don't match");
                }
                return false;
            }
            else {
                int n = this.matchBasicConstraints(x509Certificate) && this.matchKeyUsage(x509Certificate) && this.matchExtendedKeyUsage(x509Certificate) && this.matchSubjectKeyID(x509Certificate) && this.matchAuthorityKeyID(x509Certificate) && this.matchPrivateKeyValid(x509Certificate) && this.matchSubjectPublicKeyAlgID(x509Certificate) && this.matchPolicy(x509Certificate) && this.matchSubjectAlternativeNames(x509Certificate) && this.matchPathToNames(x509Certificate) && this.matchNameConstraints(x509Certificate);
                if (n && X509CertSelector.debug != null) {
                    X509CertSelector.debug.println("X509CertSelector.match returning: true");
                }
                return n;
            }
        }
    }
    
    private boolean matchSubjectKeyID(X509Certificate x509Certificate) {
        if (this.subjectKeyID == null) {
            return true;
        }
        try {
            byte[] extensionValue = x509Certificate.getExtensionValue("2.5.29.14");
            if (extensionValue == null) {
                if (X509CertSelector.debug != null) {
                    X509CertSelector.debug.println("X509CertSelector.match: no subject key ID extension");
                }
                return false;
            }
            else {
                byte[] octetString = new DerInputStream(extensionValue).getOctetString();
                if (octetString == null || !Arrays.equals(this.subjectKeyID, octetString)) {
                    if (X509CertSelector.debug != null) {
                        X509CertSelector.debug.println("X509CertSelector.match: subject key IDs don't match\nX509CertSelector.match: subjectKeyID: " + Arrays.toString(this.subjectKeyID) + "\nX509CertSelector.match: certSubjectKeyID: " + Arrays.toString(octetString));
                    }
                    return false;
                }
            }
        }
        catch (IOException ex) {
            if (X509CertSelector.debug != null) {
                X509CertSelector.debug.println("X509CertSelector.match: exception in subject key ID check");
            }
            return false;
        }
        return true;
    }
    
    private boolean matchAuthorityKeyID(X509Certificate x509Certificate) {
        if (this.authorityKeyID == null) {
            return true;
        }
        try {
            byte[] extensionValue = x509Certificate.getExtensionValue("2.5.29.35");
            if (extensionValue == null) {
                if (X509CertSelector.debug != null) {
                    X509CertSelector.debug.println("X509CertSelector.match: no authority key ID extension");
                }
                return false;
            }
            else {
                byte[] octetString = new DerInputStream(extensionValue).getOctetString();
                if (octetString == null || !Arrays.equals(this.authorityKeyID, octetString)) {
                    if (X509CertSelector.debug != null) {
                        X509CertSelector.debug.println("X509CertSelector.match: authority key IDs don't match");
                    }
                    return false;
                }
            }
        }
        catch (IOException ex) {
            if (X509CertSelector.debug != null) {
                X509CertSelector.debug.println("X509CertSelector.match: exception in authority key ID check");
            }
            return false;
        }
        return true;
    }
    
    private boolean matchPrivateKeyValid(X509Certificate x509Certificate) {
        if (this.privateKeyValid == null) {
            return true;
        }
        PrivateKeyUsageExtension privateKeyUsageExtension = null;
        try {
            privateKeyUsageExtension = (PrivateKeyUsageExtension)getExtensionObject(x509Certificate, 0);
            if (privateKeyUsageExtension != null) {
                privateKeyUsageExtension.valid(this.privateKeyValid);
            }
        }
        catch (CertificateExpiredException ex) {
            if (X509CertSelector.debug != null) {
                String string = "n/a";
                try {
                    string = privateKeyUsageExtension.get("not_after").toString();
                }
                catch (CertificateException ex2) {}
                X509CertSelector.debug.println("X509CertSelector.match: private key usage not within validity date; ext.NOT_After: " + string + "; X509CertSelector: " + this.toString());
                ex.printStackTrace();
            }
            return false;
        }
        catch (CertificateNotYetValidException ex3) {
            if (X509CertSelector.debug != null) {
                String string2 = "n/a";
                try {
                    string2 = privateKeyUsageExtension.get("not_before").toString();
                }
                catch (CertificateException ex4) {}
                X509CertSelector.debug.println("X509CertSelector.match: private key usage not within validity date; ext.NOT_BEFORE: " + string2 + "; X509CertSelector: " + this.toString());
                ex3.printStackTrace();
            }
            return false;
        }
        catch (IOException ex5) {
            if (X509CertSelector.debug != null) {
                X509CertSelector.debug.println("X509CertSelector.match: IOException in private key usage check; X509CertSelector: " + this.toString());
                ex5.printStackTrace();
            }
            return false;
        }
        return true;
    }
    
    private boolean matchSubjectPublicKeyAlgID(X509Certificate x509Certificate) {
        if (this.subjectPublicKeyAlgID == null) {
            return true;
        }
        try {
            DerValue derValue = new DerValue(x509Certificate.getPublicKey().getEncoded());
            if (derValue.tag != 48) {
                throw new IOException("invalid key format");
            }
            AlgorithmId parse = AlgorithmId.parse(derValue.data.getDerValue());
            if (X509CertSelector.debug != null) {
                X509CertSelector.debug.println("X509CertSelector.match: subjectPublicKeyAlgID = " + this.subjectPublicKeyAlgID + ", xcert subjectPublicKeyAlgID = " + parse.getOID());
            }
            if (!this.subjectPublicKeyAlgID.equals(parse.getOID())) {
                if (X509CertSelector.debug != null) {
                    X509CertSelector.debug.println("X509CertSelector.match: subject public key alg IDs don't match");
                }
                return false;
            }
        }
        catch (IOException ex) {
            if (X509CertSelector.debug != null) {
                X509CertSelector.debug.println("X509CertSelector.match: IOException in subject public key algorithm OID check");
            }
            return false;
        }
        return true;
    }
    
    private boolean matchKeyUsage(X509Certificate x509Certificate) {
        if (this.keyUsage == null) {
            return true;
        }
        boolean[] keyUsage = x509Certificate.getKeyUsage();
        if (keyUsage != null) {
            int i = 0;
            while (i < this.keyUsage.length) {
                if (this.keyUsage[i] && (i >= keyUsage.length || !keyUsage[i])) {
                    if (X509CertSelector.debug != null) {
                        X509CertSelector.debug.println("X509CertSelector.match: key usage bits don't match");
                    }
                    return false;
                }
                else {
                    ++i;
                }
            }
        }
        return true;
    }
    
    private boolean matchExtendedKeyUsage(X509Certificate x509Certificate) {
        if (this.keyPurposeSet == null || this.keyPurposeSet.isEmpty()) {
            return true;
        }
        try {
            ExtendedKeyUsageExtension extendedKeyUsageExtension = (ExtendedKeyUsageExtension)getExtensionObject(x509Certificate, 4);
            if (extendedKeyUsageExtension != null) {
                Vector<Object> value = extendedKeyUsageExtension.get("usages");
                if (!value.contains(X509CertSelector.ANY_EXTENDED_KEY_USAGE) && !value.containsAll(this.keyPurposeOIDSet)) {
                    if (X509CertSelector.debug != null) {
                        X509CertSelector.debug.println("X509CertSelector.match: cert failed extendedKeyUsage criterion");
                    }
                    return false;
                }
            }
        }
        catch (IOException ex) {
            if (X509CertSelector.debug != null) {
                X509CertSelector.debug.println("X509CertSelector.match: IOException in extended key usage check");
            }
            return false;
        }
        return true;
    }
    
    private boolean matchSubjectAlternativeNames(X509Certificate x509Certificate) {
        if (this.subjectAlternativeNames == null || this.subjectAlternativeNames.isEmpty()) {
            return true;
        }
        try {
            SubjectAlternativeNameExtension subjectAlternativeNameExtension = (SubjectAlternativeNameExtension)getExtensionObject(x509Certificate, 1);
            if (subjectAlternativeNameExtension == null) {
                if (X509CertSelector.debug != null) {
                    X509CertSelector.debug.println("X509CertSelector.match: no subject alternative name extension");
                }
                return false;
            }
            else {
                GeneralNames value = subjectAlternativeNameExtension.get("subject_name");
                Iterator<GeneralNameInterface> iterator = this.subjectAlternativeGeneralNames.iterator();
                while (iterator.hasNext()) {
                    GeneralNameInterface generalNameInterface = (GeneralNameInterface)iterator.next();
                    boolean equals = false;
                    for (Iterator iterator2 = value.iterator(); iterator2.hasNext() && equals == 0; equals = ((GeneralName)iterator2.next()).getName().equals(generalNameInterface)) {}
                    if (equals == 0 && (this.matchAllSubjectAltNames || !iterator.hasNext())) {
                        if (X509CertSelector.debug != null) {
                            X509CertSelector.debug.println("X509CertSelector.match: subject alternative name " + generalNameInterface + " not found");
                        }
                        return false;
                    }
                    else {
                        if (equals && !this.matchAllSubjectAltNames) {
                            break;
                        }
                        continue;
                    }
                }
            }
        }
        catch (IOException ex) {
            if (X509CertSelector.debug != null) {
                X509CertSelector.debug.println("X509CertSelector.match: IOException in subject alternative name check");
            }
            return false;
        }
        return true;
    }
    
    private boolean matchNameConstraints(X509Certificate cert) {
        if (this.nc == null) {
            return true;
        }
        try {
            if (!this.nc.verify(cert)) {
                if (X509CertSelector.debug != null) {
                    X509CertSelector.debug.println("X509CertSelector.match: name constraints not satisfied");
                }
                return false;
            }
        }
        catch (IOException ex) {
            if (X509CertSelector.debug != null) {
                X509CertSelector.debug.println("X509CertSelector.match: IOException in name constraints check");
            }
            return false;
        }
        return true;
    }
    
    private boolean matchPolicy(X509Certificate x509Certificate) {
        if (this.policy == null) {
            return true;
        }
        try {
            CertificatePoliciesExtension certificatePoliciesExtension = (CertificatePoliciesExtension)getExtensionObject(x509Certificate, 3);
            if (certificatePoliciesExtension == null) {
                if (X509CertSelector.debug != null) {
                    X509CertSelector.debug.println("X509CertSelector.match: no certificate policy extension");
                }
                return false;
            }
            else {
                List<Object> value = certificatePoliciesExtension.get("policies");
                ArrayList<CertificatePolicyId> list = new ArrayList(value.size());
                Iterator<Object> iterator = value.iterator();
                while (iterator.hasNext()) {
                    list.add(((PolicyInformation)iterator.next()).getPolicyIdentifier());
                }
                if (this.policy != null) {
                    boolean n = false;
                    if (this.policy.getCertPolicyIds().isEmpty()) {
                        if (list.isEmpty()) {
                            if (X509CertSelector.debug != null) {
                                X509CertSelector.debug.println("X509CertSelector.match: cert failed policyAny criterion");
                            }
                            return false;
                        }
                    }
                    else {
                        Iterator<CertificatePolicyId> iterator2 = this.policy.getCertPolicyIds().iterator();
                        while (iterator2.hasNext()) {
                            if (list.contains((Object)iterator2.next())) {
                                n = true;
                                break;
                            }
                            continue;
                        }
                        if (n == 0) {
                            if (X509CertSelector.debug != null) {
                                X509CertSelector.debug.println("X509CertSelector.match: cert failed policyAny criterion");
                            }
                            return false;
                        }
                    }
                }
            }
        }
        catch (IOException ex) {
            if (X509CertSelector.debug != null) {
                X509CertSelector.debug.println("X509CertSelector.match: IOException in certificate policy ID check");
            }
            return false;
        }
        return true;
    }
    
    private boolean matchPathToNames(X509Certificate x509Certificate) {
        if (this.pathToGeneralNames == null) {
            return true;
        }
        try {
            NameConstraintsExtension nameConstraintsExtension = (NameConstraintsExtension)getExtensionObject(x509Certificate, 2);
            if (nameConstraintsExtension == null) {
                return true;
            }
            if (X509CertSelector.debug != null && Debug.isOn("certpath")) {
                X509CertSelector.debug.println("X509CertSelector.match pathToNames:\n");
                Iterator<GeneralNameInterface> iterator = this.pathToGeneralNames.iterator();
                while (iterator.hasNext()) {
                    X509CertSelector.debug.println("    " + iterator.next() + "\n");
                }
            }
            GeneralSubtrees value = nameConstraintsExtension.get("permitted_subtrees");
            GeneralSubtrees value2 = nameConstraintsExtension.get("excluded_subtrees");
            if (value2 != null && !this.matchExcluded(value2)) {
                return false;
            }
            if (value != null && !this.matchPermitted(value)) {
                return false;
            }
        }
        catch (IOException ex) {
            if (X509CertSelector.debug != null) {
                X509CertSelector.debug.println("X509CertSelector.match: IOException in name constraints check");
            }
            return false;
        }
        return true;
    }
    
    private boolean matchExcluded(GeneralSubtrees generalSubtrees) {
        Iterator<GeneralSubtree> iterator = generalSubtrees.iterator();
        while (iterator.hasNext()) {
            GeneralNameInterface name = ((GeneralSubtree)iterator.next()).getName().getName();
            for (GeneralNameInterface obj : this.pathToGeneralNames) {
                if (name.getType() == obj.getType()) {
                    switch (obj.constrains(name)) {
                        case 0:
                        case 2: {
                            if (X509CertSelector.debug != null) {
                                X509CertSelector.debug.println("X509CertSelector.match: name constraints inhibit path to specified name");
                                X509CertSelector.debug.println("X509CertSelector.match: excluded name: " + obj);
                            }
                            return false;
                        }
                        default: {
                            continue;
                        }
                    }
                }
                else {
                    continue;
                }
            }
        }
        return true;
    }
    
    private boolean matchPermitted(GeneralSubtrees generalSubtrees) {
        for (GeneralNameInterface generalNameInterface : this.pathToGeneralNames) {
            Iterator<GeneralSubtree> iterator2 = generalSubtrees.iterator();
            int n = 0;
            boolean n2 = false;
            String string = "";
            while (iterator2.hasNext() && n == 0) {
                GeneralNameInterface name = ((GeneralSubtree)iterator2.next()).getName().getName();
                if (name.getType() == generalNameInterface.getType()) {
                    n2 = true;
                    string = string + "  " + name;
                    switch (generalNameInterface.constrains(name)) {
                        case 0:
                        case 2: {
                            n = 1;
                            continue;
                        }
                    }
                }
                else {
                    continue;
                }
            }
            if (n == 0 && n2) {
                if (X509CertSelector.debug != null) {
                    X509CertSelector.debug.println("X509CertSelector.match: name constraints inhibit path to specified name; permitted names of type " + generalNameInterface.getType() + ": " + string);
                }
                return false;
            }
            else {
                continue;
            }
        }
        return true;
    }
    
    private boolean matchBasicConstraints(X509Certificate x509Certificate) {
        if (this.basicConstraints == -1) {
            return true;
        }
        int basicConstraints = x509Certificate.getBasicConstraints();
        if (this.basicConstraints == -2) {
            if (basicConstraints != -1) {
                if (X509CertSelector.debug != null) {
                    X509CertSelector.debug.println("X509CertSelector.match: not an EE cert");
                }
                return false;
            }
        }
        else if (basicConstraints < this.basicConstraints) {
            if (X509CertSelector.debug != null) {
                X509CertSelector.debug.println("X509CertSelector.match: cert's maxPathLen is less than the min maxPathLen set by basicConstraints. (" + basicConstraints + " < " + this.basicConstraints + ")");
            }
            return false;
        }
        return true;
    }
    
    private static <T> Set<T> cloneSet(Set<T> c) {
        if ((c instanceof HashSet)) {
            return (Set)((HashSet)c).clone();
        }
        return new HashSet(c);
    }
    
    @Override
    public Object clone() {
        try {
            X509CertSelector x509CertSelector = (X509CertSelector)super.clone();
            if (this.subjectAlternativeNames != null) {
                x509CertSelector.subjectAlternativeNames = cloneSet(this.subjectAlternativeNames);
                x509CertSelector.subjectAlternativeGeneralNames = cloneSet(this.subjectAlternativeGeneralNames);
            }
            if (this.pathToGeneralNames != null) {
                x509CertSelector.pathToNames = cloneSet(this.pathToNames);
                x509CertSelector.pathToGeneralNames = cloneSet(this.pathToGeneralNames);
                return x509CertSelector;
            }
            return x509CertSelector;
        }
        catch (CloneNotSupportedException cause) {
            throw new InternalError(cause.toString(), cause);
        }
    }
    
    static {
        debug = Debug.getInstance("certpath");
        ANY_EXTENDED_KEY_USAGE = ObjectIdentifier.newInternal(new int[] { 2, 5, 29, 37, 0 });
        CertPathHelperImpl.initialize();
        FALSE = Boolean.FALSE;
        (EXTENSION_OIDS = new String[5])[0] = "2.5.29.16";
        X509CertSelector.EXTENSION_OIDS[1] = "2.5.29.17";
        X509CertSelector.EXTENSION_OIDS[2] = "2.5.29.30";
        X509CertSelector.EXTENSION_OIDS[3] = "2.5.29.32";
        X509CertSelector.EXTENSION_OIDS[4] = "2.5.29.37";
    }
}
